import json
from django.utils.html import format_html, format_html_join, mark_safe
from django.templatetags.static import static
from wagtail.core import hooks

from . import context


@hooks.register('insert_editor_js')
def yoast_panel_js():
    cxt = json.dumps({
        'version': context.VERSION,
        'locale': context.LOCALE,
    })
    js_files = [
        'wagtailyoast/dist/js/yoastworker%s.js' % context.VERSION,
        'wagtailyoast/dist/js/yoastanalysis%s.js' % context.VERSION,
    ]
    js_includes = format_html_join(
        '\n',
        '<script src="{0}"></script>',
        ((static(filename),) for filename in js_files)
    )
    js_exec = format_html(
        "<script>{}</script>",
        mark_safe(
            "$(function() {"
            "const panel = new Yoast.Panel(%s);"
            "panel.init();"
            "});" % cxt)
    )
    return js_includes + js_exec


@hooks.register('insert_editor_css')
def yoast_panel_css():
    css_files = [
        'wagtailyoast/dist/css/styles%s.css' % context.VERSION,
    ]
    css_includes = format_html_join(
        '\n',
        '<link href="{0}" rel="stylesheet" type="text/css">',
        ((static(filename),) for filename in css_files)
    )
    return css_includes



import os
import json
import pkg_resources
from django.conf import settings

# Version

try:
    #  Production part
    VERSION = pkg_resources.get_distribution("wagtailyoast").version
except pkg_resources.DistributionNotFound:
    #  Develop part
    with open(os.path.join(settings.BASE_DIR, 'package.json')) as package:
        data = json.load(package)
        VERSION = data['version']

LOCALE = settings.WY_LOCALE



import json
from django.contrib.auth.models import Group, User
from django.core.management.base import BaseCommand
from django.contrib.contenttypes.models import ContentType
from wagtail.core.models import Page, Site
from wagtail.images.models import Image

from tests.models import TestPage
from tests.blocks import TextBlock, ImageBlock
from tests import constants


class Command(BaseCommand):

    def handle(self, *args, **options):
        Site.objects.all().delete()
        Page.objects.all().delete()
        Image.objects.all().delete()

        text = TextBlock()
        image = ImageBlock()

        # Create admin user

        user, created = User.objects.get_or_create(
            username='wagtail',
            first_name='User',
            last_name='Admin',
            email='test@test.test',
            is_staff=True,
            is_superuser=True,
            is_active=True
        )
        if created:
            user.save()
            user.set_password('yoast')
            for group in Group.objects.all():
                user.groups.add(group)
            user.save()

        #  RootPage, HomePage, Site

        page_content_type, created = ContentType.objects.get_or_create(
            model='page',
            app_label='wagtailcore'
        )

        root = Page.objects.create(
            title="Root",
            slug='root',
            content_type=page_content_type,
            path='0001',
            depth=1,
            numchild=1,
            url_path='/',
        )
        root.save()

        home_page = TestPage.objects.create(
            title="Examples - Test page",
            slug='home',
            path='00010001',
            depth=2,
            numchild=0,
            body=json.dumps([
                text.mock(constants.WELCOME_TEXT),
            ])
        )
        home_page.save()

        site = Site.objects.create(
            hostname='localhost',
            port=8080,
            root_page_id=home_page.id,
            is_default_site=True
        )
        site.save()

        # Examples ------------------------------------------------------------

        # Goal : get perfect score with yoast

        content_page = TestPage(
            title="Example 1 - PerfectPage",
            seo_title="perfect yoast page",
            search_description="perfect yoast page",
            slug='perfect',
            keywords='perfect yoast page',
            body=json.dumps([
                text.mock(constants.TEXT_1),
                image.mock('perfect yoast page'),
                text.mock(constants.TEXT_1),
            ])
        )
        home_page.add_child(instance=content_page)

        content_page = TestPage(
            title="Example 2 - EmptyPage",
            seo_title="",
            search_description="",
            slug='empty',
            body=json.dumps([
            ])
        )
        home_page.add_child(instance=content_page)



import requests
from io import BytesIO
from django.core.files.images import ImageFile
from wagtail.images.blocks import ImageChooserBlock
from wagtail.core.blocks import RichTextBlock
from wagtail.images.models import Image

from . import constants


class TextBlock(RichTextBlock):
    def __init__(self, **kwargs):
        super().__init__(features=constants.RICH_TEXT_FEATURES, **kwargs)

    def mock(self, content):
        return {
            'type': 'text',
            'value': str.strip(content)
        }


class ImageBlock(ImageChooserBlock):
    def mock(self, title):
        url = str.strip(constants.URL_IMAGE_MOCK_1)
        filename = "%s.png" % title
        try:
            ret = Image.objects.get(title=title)
        except Image.DoesNotExist:
            response = requests.get(url)
            file = ImageFile(BytesIO(response.content), name=filename)
            ret = Image(
                title=title,
                file=file
            )
            import pdb
            pdb.set_trace()
            ret.save()
        return {
            'type': 'image',
            'value': ret.id
        }



RICH_TEXT_FEATURES = [
    'h1',
    'h2',
    'h3',
    'h4',
    'bold',
    'italic',
    'link',
    'ol',
    'ul',
]

URL_IMAGE_MOCK_1 = \
    """
    https://media.wagtail.io/images/w2.original.original_Xk3v8T2.width-700.png
    """

TEXT_1 = \
    """
    <h1>This page is rated perfect with Wagtail Yoast SEO</h1>
    <p>
    Lorem ipsum dolor sit amet, consectetur adipiscing elit,
    sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.
    Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris
    nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in
    reprehenderit in voluptate velit esse cillum dolore eu
    fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident,
    sunt in culpa qui officia deserunt mollit anim id est laborum.
    <p>
    """

WELCOME_TEXT = \
    """
    <h1>Welcome on Wagtail Yoast SEO</h1>
    <h2>
    Log in
    <a href="http://localhost:8080/wagtail/" target="_blank">
    Wagtail Admin
    </a>
    with :
    </h2>
    <p>Username: <kbd>wagtail</kbd></p>
    <p>Password: <kbd>yoast</kbd></p>
    """



# Generated by Django 3.0.9 on 2020-08-30 09:17

from django.db import migrations, models
import django.db.models.deletion
import tests.blocks
import wagtail.core.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0052_pagelogentry'),
    ]

    operations = [
        migrations.CreateModel(
            name='TestPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
                ('body', wagtail.core.fields.StreamField([('text', tests.blocks.TextBlock()), ('image', tests.blocks.ImageBlock())], blank=True)),
                ('keywords', models.CharField(blank=True, default='', max_length=100)),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page',),
        ),
    ]



from django import forms
from wagtail.admin.edit_handlers import ObjectList
from wagtail.admin.edit_handlers import FieldPanel, MultiFieldPanel


class YoastPanel(ObjectList):
    template = "wagtailyoast/edit_handlers/yoast_panel.html"

    def __init__(self, keywords='keywords', title='seo_title',
                 search_description='search_description', slug='slug',
                 heading='Yoast', *args, **kwargs):
        #  TODO: Test if fields exist

        self.title_field = title
        self.search_description = search_description
        self.slug = slug

        children = [
            MultiFieldPanel([
                FieldPanel(
                    keywords,
                    widget=forms.TextInput(attrs={'id': 'yoast_keywords'})
                ),
            ], heading="Page")
        ]
        super().__init__(children=children, heading=heading)



from django.db import models
from django.utils.translation import gettext_lazy
from wagtail.core.fields import StreamField
from wagtail.core.models import Page
from wagtail.admin.edit_handlers import StreamFieldPanel
from wagtail.admin.edit_handlers import TabbedInterface, ObjectList

from wagtailyoast.edit_handlers import YoastPanel

from .blocks import TextBlock, ImageBlock


class TestPage(Page):
    body = StreamField([
        ('text', TextBlock()),
        ('image', ImageBlock()),
    ], blank=True)
    keywords = models.CharField(default='', blank=True, max_length=100)

    edit_handler = TabbedInterface([
        ObjectList(
            Page.content_panels + [StreamFieldPanel('body')],
            heading=gettext_lazy('Content')
        ),
        ObjectList(Page.promote_panels, heading=gettext_lazy('Promotion')),
        ObjectList(Page.settings_panels, heading=gettext_lazy('Settings')),
        YoastPanel(
            keywords='keywords',
            title='seo_title',
            search_description='search_description',
            slug='slug'
        ),
    ])
